//+------------------------------------------------------------------+
//|                                                 SOM_Net_Base.mqh |
//|                                            Copyright 2015, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <SOM_Net_Base.mqh>                                      |
//+------------------------------------------------------------------+
#include "SOMNode.mqh" 
#define DIMENSION_HEADER 5
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

struct Conversion_Double { double d; };
struct Conversion_UChar  { uchar  c[8];};
Conversion_Double c_double[];
Conversion_UChar  c_uchar[];
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSOM_Net_Base
  {
protected:
   //---  
   int               m_xcells;
   int               m_ycells;
   //---  BMP-
   int               m_xsize;
   int               m_ysize;
   //---
   int               m_total_nodes;
   CSOMNode          m_som_nodes[];
   //---
   int               m_dimension;
   string            m_som_titles[];
   bool              m_byte_map[];
public:
   //---    ,        
   CSOMNode         *public_node;
                     CSOM_Net_Base(void){};
                    ~CSOM_Net_Base(void){};
   //---    ind     public_node
   void              GetNode(int ind){ public_node=m_som_nodes[ind].GetObjPointer(); };
   //---    
   int               GetDimension(){return(m_dimension);};
   //---                      
   bool              DownloadNet(string file_name);
   //---           
   int               BestMatchingNode(const double &vector[]);
   //---        ,     dimension
   int               BestMatchingNode(const double &vector[],int dimension);
   //---           (     )
   bool              InitSetByteMap(int num,bool value);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSOM_Net_Base::DownloadNet(string file_name)
  {
   double downl_net[];
//---   
   ResetLastError();
   int hndl=FileOpen(file_name+".somnet",FILE_BIN|FILE_READ);
   if(hndl!=INVALID_HANDLE)
     {
      int length = FileReadInteger(hndl);
      FileReadArray(hndl, downl_net, 0, length);
     }
   else
     {
      m_dimension=0;
      Print("Error open file "+file_name+".somnet , err="+(string)GetLastError());
      return(0);
     }
//---
   int total_downl=ArraySize(downl_net);
   int size_net,shift_index;
//---  Copy Hendle Net
   int cnt=0;
   if(total_downl>DIMENSION_HEADER)
     {
      shift_index = (int)NormalizeDouble(downl_net[cnt],0);     cnt++;
      m_xcells    = (int)NormalizeDouble(downl_net[cnt],0);     cnt++;
      m_ycells    = (int)NormalizeDouble(downl_net[cnt],0);     cnt++;
      m_xsize     = (int)NormalizeDouble(downl_net[cnt],0);     cnt++;
      m_ysize     = (int)NormalizeDouble(downl_net[cnt],0);     cnt++;
      //---
      size_net=m_xcells*m_ycells;
      m_dimension=shift_index-6;
      //--- Copy nodes
      double temp_node[]; ArrayResize(temp_node,shift_index);
      m_total_nodes=ArrayResize(m_som_nodes,size_net);
      for(int i=0;i<size_net;i++)
        {
         for(int j=0;j<shift_index;j++)
           {
            if(cnt<total_downl){temp_node[j]=downl_net[cnt]; cnt++;}
           }
         m_som_nodes[i].DownloadNode(temp_node);
        }
      //---     double 
      /*
      int count_double=ArrayResize(c_double,total_downl-cnt);
      for(int i=0;i<count_double;i++)
        {
         c_double[i].d=downl_net[cnt]; cnt++;
        }
      //---
      ArrayResize(c_uchar,count_double);
      uchar txtdownl[]; ArrayResize(txtdownl,count_double*8);
      int dcnt=0;
      for(int i=0;i<count_double;i++)
        {
         c_uchar[i]=(Conversion_UChar)c_double[i];
         for(int j=0;j<8;j++) {txtdownl[dcnt]=c_uchar[i].c[j]; dcnt++;}
        }
      string txt_titles=CharArrayToString(txtdownl);  Print(txt_titles);
      //---
      ushort separator[]; StringToShortArray(";",separator);
      //---
      m_total_nodes=StringSplit(txt_titles,separator[0],m_som_titles);
      */
      int dim = FileReadInteger(hndl);
      if(m_dimension != dim)
      {
        Print("Incorrect titles: ", m_dimension, " vs ", dim);
      }
      ArrayResize(m_som_titles, dim);
      for(int i = 0; i < dim; i++)
      {
        int len = FileReadInteger(hndl);
        m_som_titles[i] = FileReadString(hndl, len);
      }
      FileClose(hndl);
     }
   else
     {
      Print("file does not contain data SOM_Net");
      FileClose(hndl);
      return(0);
     }
   return(1);
  };
//+------------------------------------------------------------------+
//|                              |
//+------------------------------------------------------------------+
int CSOM_Net_Base::BestMatchingNode(const double &vector[],int dimension)
  {
   int    min_ind=0; int dim=(dimension<m_dimension?dimension:m_dimension);
   double min_dist=m_som_nodes[min_ind].CalculateDistance(vector,dim);
   int total_nodes= ArraySize(m_som_nodes);
   for(int i=0; i<total_nodes; i++)
     {
      double temp_dist=m_som_nodes[i].CalculateDistance(vector,dim);
      if(temp_dist<min_dist)
        {
         min_dist=temp_dist;
         min_ind=i;
        };
     }
   return(min_ind);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CSOM_Net_Base::BestMatchingNode(const double &vector[])
  {
   int    min_ind=0; int dim=m_dimension;
//--- ,   ,      
   if(ArraySize(m_byte_map)<m_dimension)InitSetByteMap(0,1);
//---
   double min_dist=m_som_nodes[min_ind].CalculateDistance(vector,m_byte_map);
   int total_nodes= ArraySize(m_som_nodes);
   for(int i=0; i<total_nodes; i++)
     {
      double temp_dist=m_som_nodes[i].CalculateDistance(vector,m_byte_map);
      if(temp_dist<min_dist)
        {
         min_dist=temp_dist;
         min_ind=i;
        };
     }
   return(min_ind);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSOM_Net_Base::InitSetByteMap(int num,bool value)
  {
   int size=ArraySize(m_byte_map);
//---      
   if(size<m_dimension)
     {
      ArrayResize(m_byte_map,m_dimension);
      ArrayInitialize(m_byte_map,1);
     }
//---   
   if(num>=size || num<0)return(0);
//---  
   m_byte_map[num]=value;
   return(1);
  };
//+------------------------------------------------------------------+
